'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.createResolver = undefined;

var _helpers = require('./../helpers');

var _createHooks = require('./../createHooks');

var createResolver = exports.createResolver = function createResolver() {
  var startHooks = [];
  var beforeTransitionHooks = [];
  var afterTransitionHooks = [];
  var finalHooks = [];

  var addStartHook = function addStartHook(hook) {
    (0, _createHooks.validateHook)(hook);
    startHooks.push(hook);
  };
  var addBeforeTransitionHook = function addBeforeTransitionHook(hook) {
    (0, _createHooks.validateHook)(hook);
    beforeTransitionHooks.push(hook);
  };
  var addAfterTransitionHook = function addAfterTransitionHook(hook) {
    (0, _createHooks.validateHook)(hook);
    afterTransitionHooks.push(hook);
  };
  var addFinalHook = function addFinalHook(hook) {
    (0, _createHooks.validateHook)(hook);
    finalHooks.push(hook);
  };

  var getHooks = function getHooks(transition) {
    return startHooks.concat(beforeTransitionHooks, [transition], afterTransitionHooks, finalHooks);
  };

  var triggerHooks = function triggerHooks(components, attrs) {
    var transition = arguments.length <= 2 || arguments[2] === undefined ? function () {} : arguments[2];

    return getHooks(transition).reduce(function (promise, next) {
      var nextPromise = undefined;
      if ((0, _createHooks.isLocalHook)(next)) {
        nextPromise = (0, _helpers.getDependencies)(components, next.name, attrs);
      } else if ((0, _createHooks.isGlobalHook)(next)) {
        nextPromise = next.callback.bind(null, attrs);
      }
      return promise.then(nextPromise);
    }, Promise.resolve());
  };

  return {
    addStartHook: addStartHook,
    addBeforeTransitionHook: addBeforeTransitionHook,
    addAfterTransitionHook: addAfterTransitionHook,
    addFinalHook: addFinalHook,
    triggerHooks: triggerHooks
  };
};